/***************************************************************************
                          eqband.h  -  description
                             -------------------
    begin                : Mon Jun 16 20:12:07 CEST 2006
    copyright            : (C) 2006 by Nicola Doebelin
    email                : nicola@doebelin.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QList>
#include <QPointF>
#include <QString>

class EqBand
{
  public:
    EqBand(int);
    ~EqBand();

    void setFreq(float f)       { _freq = f; };
    void setGain(float f)       { _gain = f; };
    void setBandwidth(float f)  { _bandwidth = f; };
    void setActive(bool b)      { _active = b; };
    void toggleActive()         { _active = !_active; };
    void setSamplerate(float f) { _samplerate = f; };
    void setMode(int i)         { _mode = i; };

    float gain()        { return _gain; };
    float freq()        { return _freq; };
    float bandwidth()   { return _bandwidth; };
    bool isActive()     { return _active; };
    int id()            { return _id; };
    float samplerate()  { return _samplerate; };
    int mode()          { return _mode; };
    QString modeString();

    void calcFilterCoeffs();

    void calcFilterCoeffs(int const,    // type
                          double const, // frequency
                          double const, // sample rate
                          double const, // q
                          double const, // gain (dB)
                          bool);        // is q bandwidth?

    float filter(float);    // sample

    void calcCurve(QList<QPointF> &);

  private:
    float _freq,
          _gain,
          _bandwidth,
          _samplerate;
    bool _active;
    int _id,
        _mode;

    // filter coeffs
    float b0a0,b1a0,b2a0,a1a0,a2a0;

    // in/out history
    float ou1,ou2,in1,in2;

};

/* EOF */
